/* Copyright 2013 Daniel Hanrahan. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE DANIEL HANRAHAN "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL DANIEL HANRAHAN BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef EDIFICE_HPP
#define EDIFICE_HPP

#include <map>
#include <vector>
#include <stdexcept>
#include "Building.hpp"

class BofReader;
class BofWriter;
class FilePatcher;

class Edifice {
public:
	Edifice (const std::string& BofLocation);

	void patch();

	size_t buildingCount() const { return m_Buildings.size(); }

	size_t getTechLevels () const { return m_TechLevels; }
	void setTechLevels (size_t techLevels) { m_TechLevels = techLevels; }
	void resetTechLevels () { m_TechLevels = 11; }

	int getMinBuildingTechLevel() const;
	int getMaxBuildingTechLevel() const;

	int getMinTechShift () const;
	int getMaxTechShift () const;
	void shiftTechRequirements (int change);

private:
	const std::string m_BofLocation;
	const std::string m_StBofRes;
	const std::string m_TrekExe;

	void read (BofReader& r);
	void write (BofWriter& w);

	void patchTrekExe (FilePatcher& p);
	void patchMaxTechLevel (FilePatcher& p);
	void patchMaxBuildingNumber (FilePatcher& p);

	size_t computeTechMapSize (size_t FileSize, size_t NumBuildings) const;
	size_t computeTechLevelsFromTechMapSize (const size_t TechMapSize) const;

	size_t getTechMapSize () const {
		return 16 + 7 * (m_TechLevels * 6 + 2);
	}
	void writeTechMap (BofWriter& w);

	size_t getIgnoreListsSize () const { return 240; }
	void writeIgnoreLists (BofWriter& w);

	size_t getBuildingsOffset () const {
		return 8 + getTechMapSize() + getIgnoreListsSize();
	}
	void readBuildings (BofReader& r, size_t numBuildings);
	void writeBuildings (BofWriter& w);
	void sortBuildings ();

	size_t getMapAddress () const {
		return 8 + getTechMapSize() + getIgnoreListsSize() + 
			m_Buildings.size() * 0xB0;
	}
	void computeBuildingAddresses ();
	void writeBuildingAddresses (BofWriter& w);

	Building& getBuildingById (int id);


	size_t m_TechLevels;

	std::vector<Building> m_Buildings;
	std::vector<size_t> m_BuildingAddresses;
//	std::vector<char> m_IgnoreLists;
};

#endif
