/* Copyright 2013 Daniel Hanrahan. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE DANIEL HANRAHAN "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL DANIEL HANRAHAN BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef BOF_FILE_READER_H
#define BOF_FILE_READER_H

#include <cstdio>
#include <cerrno>
#include <stdexcept>
#include "BofReader.hpp"

class FileReader : public BofReader {

	FILE *  fp;
	size_t  m_Size;

	void file_error (const std::string& func) const {
		throw std::runtime_error(func + ": " + strerror(errno));
	}

	/// Implements BofReader::read()
	void read (void *dest, size_t n) {
		if( fread(dest, n, 1, fp) != 1 ) file_error("read");
	}

public:
	FileReader (const std::string& filename) {
		fp = fopen(filename.c_str(), "rb");
		if (!fp) file_error("open");

		// Get the file size.
		fseek(fp, 0, SEEK_END);
		m_Size = ftell(fp);
		rewind(fp);
	}

	~FileReader () {
		fclose(fp);
	}

	/// How big is the buffer?
	size_t size() const {
		return m_Size;
	}

	/// How far are we from the start of the buffer?
	size_t position() const {
		long pos = ftell(fp);
		if (pos == -1) file_error("position");
		return pos;
	}

	/// Set the position in the buffer.
	void seek (size_t position) {
		if( fseek(fp, position, SEEK_SET) != 0 ) file_error("seek");
	}

	/// Skip a number of bytes from the current position.
	void skip (size_t num_bytes) {
		if( fseek(fp, num_bytes, SEEK_CUR) != 0 ) file_error("skip");
	}

	/// Are we at the end of the buffer?
/*	bool eof() const {
		return (bool)feof(fp);
	}*/

};

#endif

