/* Copyright 2013 Daniel Hanrahan. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE DANIEL HANRAHAN "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL DANIEL HANRAHAN BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef BOF_FILE_WRITER_H
#define BOF_FILE_WRITER_H

#include <cstdio>
#include <cerrno>
#include <stdexcept>
#include "BofWriter.hpp"

class FileWriter : public BofWriter {

	FILE *  fp;

	void file_error (const std::string& func) const {
		throw std::runtime_error(func + ": " + strerror(errno));
	}

	/// Implements BofWriter::write()
	void write (const void *data, size_t n) {
		if( fwrite(data, n, 1, fp) != 1 ) file_error("write");
	}

public:
	FileWriter (const std::string& filename) {
		fp = fopen(filename.c_str(), "wb");
		if (!fp) file_error("open");
	}

	~FileWriter () {
		fclose(fp);
	}

	/// How big is the buffer?
/*	size_t size() const {
		return position();
	}*/

	/// \name Positioning:
	/// \{

	/// How far are we from the start of the buffer?
	size_t position() const {
		long pos = ftell(fp);
		if (pos == -1) file_error("position");
		return pos;
	}

	/// Set the position in the buffer.
	void seek (size_t position) {
		if( fseek(fp, position, SEEK_SET) != 0 ) file_error("seek");
	}

	/// \}

};

#endif

