/* Copyright 2013 Daniel Hanrahan. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE DANIEL HANRAHAN "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL DANIEL HANRAHAN BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "TechMap.hpp"

TechMap::TechMap (const size_t NumTechLevels) : 
	m_TechLevels( NumTechLevels )
{
	// Init tech level building counts.
	m_Fields.resize(7);
	for (size_t fld = 0; fld < 7; ++fld) {
		m_Fields[fld].m_Levels.resize (NumTechLevels);
	}
}

size_t TechMap::Field::buildingCount() const {
	size_t total = 0;
	for (size_t lvl = 0; lvl < m_Levels.size(); ++lvl) {
		total += m_Levels[lvl].buildingCount;
	}
	return total;
}

void TechMap::write (BofWriter& w)
{
	// Tech field building counts:
	for (size_t fld = 0; fld < 7; ++fld) {
		w.write_word (field(fld).buildingCount());
	}
	w.write_word (0); // (padding)

	// Tech level building addresses and counts:
	for (size_t fld = 0; fld < 7; ++fld) {
		// Building addresses:
		for (size_t lvl = 0; lvl < m_TechLevels; ++lvl) {
			w.write_dword (field(fld).level(lvl).startAddress);
		}
		// Building counts:
		for (size_t lvl = 0; lvl < m_TechLevels; ++lvl) {
			w.write_word (field(fld).level(lvl).buildingCount);
		}
		w.write_word (0); // (padding)
	}
}

