/* Copyright 2013 Daniel Hanrahan. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE DANIEL HANRAHAN "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL DANIEL HANRAHAN BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef BOF_ZIPFILE_HPP
#define BOF_ZIPFILE_HPP

#include "Archive.hpp"

// Forward references.
struct zip;

/// A class for reading/writing files from a ZIP archive.
class ZipFile : public Archive {
public:
	ZipFile (const std::string& zipfile);

// Reading archive:

	/// Open a file for reading.
	boost::shared_ptr<BofReader>  open_read (const std::string& name);

// Modifying archive:

	/// Open a file for writing.
	boost::shared_ptr<BofWriter>  open_write (const std::string& name);

private:
	const std::string m_ZipFile;  // filename of Zip archive.

	class ZipReader; // subclass of BofReader
	class ZipWriter; // subclass of BofWriter

	// Custom deleters for the above classes.
	static void ZipReaderDeleter (BofReader *br);
	static void ZipWriterDeleter (BofWriter *bw);
};

#endif

